from langchain.output_parsers import PydanticOutputParser
from langchain.prompts import PromptTemplate
from parser_classes import task_classes

class LLMTask:
    def __init__(self, llm, prompt_attr):
        self.llm = llm
        self.prompt_attr = prompt_attr
        self.prompt = None
        self.parser = None
        
    def build_prompt(self, given_partial_variables={}):
        partial_variables = {}
        format_instructions_str = 'format_instructions'
        if self.prompt_attr.task_name:
            self.parser = PydanticOutputParser(pydantic_object=task_classes[self.prompt_attr.task_name])

        for pv in self.prompt_attr.required_partial_variables:
            if pv == format_instructions_str:
                if not self.parser:
                    raise Exception
                else:
                    partial_variables[format_instructions_str] = self.parser.get_format_instructions()
            elif pv not in given_partial_variables.keys():
                raise Exception
            else:
                partial_variables[pv] = given_partial_variables[pv]

        self.prompt = PromptTemplate(
                template=self.prompt_attr.text,
                input_variables=self.prompt_attr.required_input_variables,
                partial_variables = partial_variables
            )
        
    def get_chain(self):
        if not self.prompt:
            self.build_prompt()
        return self.prompt | self.llm | self.parser



import os
from yaml import safe_load

class PromptManagerV2():
  """ PromptManager
  Class to contain a collection of prompts. The prompts are loaded from a YAML file
  and saved in self.versions.

  Utility methods are provided, such as latest(), to retrieve the last version.
  """
  current_script_dir = os.path.dirname(os.path.abspath(__file__))
  prompt_yaml_file_path = os.path.join(current_script_dir, "prompts")

  def __init__(self, prompts_dir = prompt_yaml_file_path):
    self.prompts_dir = prompts_dir
    self.prompts_dict = self.__get_yaml_files_dict()
    
  def get_prompt_attributes(self, task_name, version = "latest"):
    with open(self.prompts_dict[task_name], 'r') as f:
      prompts = safe_load(f)

    if version == "latest":
      version = max([k for k in list(prompts.keys()) if isinstance(k) == float])

    return PromptAttributes(
      text = prompts[version]['text'],
      version = version,
      required_input_variables = prompts[version]['input_variables'],
      required_partial_variables = prompts[version]['required_partial_variables'],
      task_name=task_name
      )
      
  def __get_yaml_files_dict(self):
    yaml_files_dict = {}
    for filename in os.listdir(self.prompts_dir):
        if filename.endswith(".yaml"):
            key = filename.split('.yaml')[0]
            absolute_path = os.path.abspath(os.path.join(self.prompts_dir, filename))
            yaml_files_dict[key] = absolute_path
    return yaml_files_dict

  def display_versions(self, markdown=False):
    string = ''
    if markdown:
      for version,prompt in self.versions.items():
        string += f'## Version {version}\n'
        string += prompt.replace('\n','\n\n') + '\n'
        string += 100*'-' + '\n'
    else:
      for version,prompt in self.versions.items():
        string += f'  Version {version}\n'
        string += 20*'-' + '\n'
        string += prompt + '\n'
        string += 100*'-' + '\n'
      print(string)


class PromptAttributes():
  def __init__(self, text, version, required_input_variables, required_partial_variables, task_name=None):
    self.version = version
    self.text = text
    self.required_input_variables = required_input_variables
    self.required_partial_variables = required_partial_variables
    self.task_name = task_name




from langchain.pydantic_v1 import BaseModel, Field

class Toxicity(BaseModel):
    answer: int = Field(description="Puntuación de toxicidad del usuario")


class Comprehension(BaseModel):
    answer: int = Field(description="Puntuación de comprension del asistente")


class Resolution(BaseModel):
    answer: int = Field(description="Puntuación de resolucion del asistente")


class Summary(BaseModel):
    answer: str = Field(description="Resumen de la siguiente conversacion")

class RatingMetrics(BaseModel):
    toxicity: int = Field(description="Puntuación de toxicidad del usuario")
    comprehension: int = Field(description="Puntuación de comprension del asistente")
    resolution: int = Field(description="Puntuación de resolucion del asistente")

class Title(BaseModel):
    answer: str = Field(description="Titulo de la conversación")

task_classes = {
    "summarization" : Summary,
    "rating_toxicity" : Toxicity,
    "rating_comprehension": Comprehension,
    "rating_resolution": Resolution,
    "rating_metrics": RatingMetrics
}


import os
import yaml

def configure(model_name='gpt-35-turbo'):
  current_script_dir = os.path.dirname(os.path.abspath(__file__))
  yaml_file_path = os.path.join(current_script_dir, "catalog", "openai_config.yaml")
  with open(yaml_file_path, "r") as file:
      openai_api_config = yaml.safe_load(file)

  os.environ["OPENAI_API_KEY"] = get_openai_key(model_name)
  os.environ["OPENAI_API_VERSION"] = openai_api_config[model_name]["OPENAI_API_VERSION"]
  os.environ["AZURE_OPENAI_ENDPOINT"] = openai_api_config[model_name]["AZURE_OPENAI_ENDPOINT"]
  
def get_openai_key(model_name):
  if 'DATABRICKS_RUNTIME_VERSION' in os.environ:
    from databricks.sdk.runtime import dbutils
    if model_name == 'gpt-35-turbo':
      return dbutils.secrets.get(scope="gen_ai", key="openai_key")
    elif model_name == 'gpt-4-1106-preview':
      return dbutils.secrets.get(scope="gen_ai", key="openai_key_gpt4")
    else:
      print(f'Model "{model_name}" not found. Please use one of: gpt-35-turbo, gpt-4-1106-preview')
      return

  else:
    import catalog.key as key
    return key.OPEN_AI_KEY(model_name)
  